package com.ruoyi.system.controller.user;

import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.InnerAuth;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysDept;
import com.ruoyi.system.api.domain.SysRole;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysPermissionService;
import com.ruoyi.system.service.ISysPostService;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import java.io.IOException;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping({"/user"})
public class SysUserController extends BaseController {
   @Autowired
   private ISysUserService userService;
   @Autowired
   private ISysRoleService roleService;
   @Autowired
   private ISysDeptService deptService;
   @Autowired
   private ISysPostService postService;
   @Autowired
   private ISysPermissionService permissionService;
   @Autowired
   private ISysConfigService configService;

   @RequiresPermissions({"system:user:list"})
   @GetMapping({"/list"})
   public TableDataInfo list(SysUser user) {
      this.startPage();
      List<SysUser> list = this.userService.selectUserList(user);
      return this.getDataTable(list);
   }

   @GetMapping({"/selectList"})
   public AjaxResult selectList(SysUser user) {
      return AjaxResult.success(this.userService.selectList(user));
   }

   @Log(
      title = "用户管理",
      businessType = BusinessType.EXPORT
   )
   @RequiresPermissions({"system:user:export"})
   @PostMapping({"/export"})
   public void export(HttpServletResponse response, SysUser user) {
      List<SysUser> list = this.userService.selectUserList(user);
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      util.exportExcel(response, list, "用户数据");
   }

   @Log(
      title = "用户管理",
      businessType = BusinessType.IMPORT
   )
   @RequiresPermissions({"system:user:import"})
   @PostMapping({"/importData"})
   public AjaxResult importData(MultipartFile file, boolean updateSupport) throws Exception {
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      List<SysUser> userList = util.importExcel(file.getInputStream());
      String operName = SecurityUtils.getUsername();
      String message = this.userService.importUser(userList, updateSupport, operName);
      return this.success(message);
   }

   @PostMapping({"/importTemplate"})
   public void importTemplate(HttpServletResponse response) throws IOException {
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      util.importTemplateExcel(response, "用户数据");
   }

   @InnerAuth
   @GetMapping({"/info/{username}"})
   public R<LoginUser> info(@PathVariable("username") String username) {
      SysUser sysUser = this.userService.selectUserByUserName(username);
      if (StringUtils.isNull(sysUser)) {
         return R.fail("用户名或密码错误");
      } else {
         Set<String> roles = this.permissionService.getRolePermission(sysUser);
         Set<String> permissions = this.permissionService.getMenuPermission(sysUser);
         LoginUser sysUserVo = new LoginUser();
         sysUserVo.setSysUser(sysUser);
         sysUserVo.setRoles(roles);
         sysUserVo.setPermissions(permissions);
         return R.ok(sysUserVo);
      }
   }

   @InnerAuth
   @PostMapping({"/register"})
   public R<Boolean> register(@RequestBody SysUser sysUser) {
      String username = sysUser.getUserName();
      if (!"true".equals(this.configService.selectConfigByKey("sys.account.registerUser"))) {
         return R.fail("当前系统没有开启注册功能！");
      } else {
         return !this.userService.checkUserNameUnique(sysUser) ? R.fail("保存用户'" + username + "'失败，注册账号已存在") : R.ok(this.userService.registerUser(sysUser));
      }
   }

   @GetMapping({"getInfo"})
   public AjaxResult getInfo() {
      SysUser user = this.userService.selectUserById(SecurityUtils.getUserId());
      Set<String> roles = this.permissionService.getRolePermission(user);
      Set<String> permissions = this.permissionService.getMenuPermission(user);
      AjaxResult ajax = AjaxResult.success();
      ajax.put("user", user);
      ajax.put("roles", roles);
      ajax.put("permissions", permissions);
      return ajax;
   }

   @RequiresPermissions({"system:user:query"})
   @GetMapping({"/", "/{userId}"})
   public AjaxResult getInfo(@PathVariable(value = "userId",required = false) Long userId) {
      this.userService.checkUserDataScope(userId);
      AjaxResult ajax = AjaxResult.success();
      List<SysRole> roles = this.roleService.selectRoleAll();
      ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter((r) -> {
         return !r.isAdmin();
      }).collect(Collectors.toList()));
      ajax.put("posts", this.postService.selectPostAll());
      if (StringUtils.isNotNull(userId)) {
         SysUser sysUser = this.userService.selectUserById(userId);
         ajax.put("data", sysUser);
         ajax.put("postIds", this.postService.selectPostListByUserId(userId));
         ajax.put("roleIds", sysUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
      }

      return ajax;
   }

   @RequiresPermissions({"system:user:add"})
   @Log(
      title = "用户管理",
      businessType = BusinessType.INSERT
   )
   @PostMapping
   public AjaxResult add(@Validated @RequestBody SysUser user) {
      this.deptService.checkDeptDataScope(user.getDeptId());
      this.roleService.checkRoleDataScope(user.getRoleIds());
      if (!this.userService.checkUserNameUnique(user)) {
         return this.error("新增用户'" + user.getUserName() + "'失败，登录账号已存在");
      } else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
         return this.error("新增用户'" + user.getUserName() + "'失败，手机号码已存在");
      } else if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
         return this.error("新增用户'" + user.getUserName() + "'失败，邮箱账号已存在");
      } else {
         user.setCreateBy(SecurityUtils.getUserId().toString());
         user.setCreateName(SecurityUtils.getLoginUser().getUsername());
         user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
         return this.toAjax(this.userService.insertUser(user));
      }
   }

   @RequiresPermissions({"system:user:edit"})
   @Log(
      title = "用户管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping
   public AjaxResult edit(@Validated @RequestBody SysUser user) {
      this.userService.checkUserAllowed(user);
      this.userService.checkUserDataScope(user.getUserId());
      this.deptService.checkDeptDataScope(user.getDeptId());
      this.roleService.checkRoleDataScope(user.getRoleIds());
      if (!this.userService.checkUserNameUnique(user)) {
         return this.error("修改用户'" + user.getUserName() + "'失败，登录账号已存在");
      } else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
         return this.error("修改用户'" + user.getUserName() + "'失败，手机号码已存在");
      } else if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
         return this.error("修改用户'" + user.getUserName() + "'失败，邮箱账号已存在");
      } else {
         user.setUpdateBy(SecurityUtils.getUsername());
         return this.toAjax(this.userService.updateUser(user));
      }
   }

   @RequiresPermissions({"system:user:remove"})
   @Log(
      title = "用户管理",
      businessType = BusinessType.DELETE
   )
   @DeleteMapping({"/{userIds}"})
   public AjaxResult remove(@PathVariable Long[] userIds) {
      return ArrayUtils.contains(userIds, SecurityUtils.getUserId()) ? this.error("当前用户不能删除") : this.toAjax(this.userService.deleteUserByIds(userIds));
   }

   @RequiresPermissions({"system:user:resetPwd"})
   @Log(
      title = "用户管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping({"/resetPwd"})
   public AjaxResult resetPwd(@RequestBody SysUser user) {
      this.userService.checkUserAllowed(user);
      this.userService.checkUserDataScope(user.getUserId());
      user.setUpdateBy(SecurityUtils.getUsername());
      return this.toAjax(this.userService.resetPwd(user));
   }

   @RequiresPermissions({"system:user:resetPwd"})
   @Log(
      title = "用户管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping({"/changeStatus"})
   public AjaxResult changeStatus(@RequestBody SysUser user) {
      this.userService.checkUserAllowed(user);
      this.userService.checkUserDataScope(user.getUserId());
      user.setUpdateBy(SecurityUtils.getUsername());
      return this.toAjax(this.userService.updateUserStatus(user));
   }

   @RequiresPermissions({"system:user:query"})
   @GetMapping({"/authRole/{userId}"})
   public AjaxResult authRole(@PathVariable("userId") Long userId) {
      AjaxResult ajax = AjaxResult.success();
      SysUser user = this.userService.selectUserById(userId);
      List<SysRole> roles = this.roleService.selectRolesByUserId(userId);
      ajax.put("user", user);
      ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter((r) -> {
         return !r.isAdmin();
      }).collect(Collectors.toList()));
      return ajax;
   }

   @RequiresPermissions({"system:user:edit"})
   @Log(
      title = "用户管理",
      businessType = BusinessType.GRANT
   )
   @PutMapping({"/authRole"})
   public AjaxResult insertAuthRole(Long userId, Long[] roleIds) {
      this.userService.checkUserDataScope(userId);
      this.roleService.checkRoleDataScope(roleIds);
      this.userService.insertUserAuth(userId, roleIds);
      return this.success();
   }

   @RequiresPermissions({"system:user:list"})
   @GetMapping({"/deptTree"})
   public AjaxResult deptTree(SysDept dept) {
      return this.success(this.deptService.selectDeptTreeList(dept));
   }
}
